<?php
class TableBlockExport_ExportToCSV_Action extends Vtiger_Action_Controller{
    var $picklistValues = array();
    function checkPermission(Vtiger_Request $request) {
        return;
    }
    function process(Vtiger_Request $request)
    {
        global $adb;
        $tableBlockId = $request->get('block_id');
        $sourceModule = $request->get('source_module');
        if($tableBlockId && $sourceModule){
            $fields= $this->getFieldsOfBlock($tableBlockId,$sourceModule);
            $translated_fields_array = array();
            $translated_fields_array[] ='PrimaryRecordName';
            foreach ($fields as $fieldId =>$fieldModel){
                $translated_fields_array[] = getTranslatedString($fieldModel->get('label'),$sourceModule);
            }
            $translated_fields_array[] = 'ID';
            $listFieldsName = array_keys($fields);
            $header = implode("\",\"",array_values($translated_fields_array));
            $header = "\"" .$header;
            $header .= "\"\r\n";
            $query = "SELECT TBD.*,CRM.label FROM table_block_data AS TBD
                        INNER JOIN vtiger_crmentity AS CRM ON TBD.crmid = CRM.crmid
                        WHERE CRM.deleted = 0 AND CRM.setype = ? AND TBD.fieldname IN (".generateQuestionMarks($listFieldsName).")
                        ORDER BY TBD.id ASC,TBD.crmid ASC";
            $rs = $adb->pquery($query,array($sourceModule,$listFieldsName));
            $data = array();
            while ($rowData = $adb->fetchByAssoc($rs)){
                $crmId = $rowData['crmid'];
                $row=$rowData['row'];
                $fieldName = $rowData['fieldname'];
                $value =$this->sanitizeValue($rowData['value'],$fields[$fieldName],$fieldName);
                $data[$crmId.'-'.$row][$fieldName] = $value;
                $data[$crmId.'-'.$row]['#_crm_label_#'] = $rowData['label'];
                if(!$data[$crmId.'-'.$row]['ID']){
                    $data[$crmId.'-'.$row]['ID'] = array($rowData['id']);
                }else{
                    array_push($data[$crmId.'-'.$row]['ID'],$rowData['id']);
                }
            }

            //process export
            $moduleName = getTranslatedString($sourceModule, $sourceModule);
            $moduleName = str_replace(" ","_",$moduleName);
            $fileName = $moduleName."-TableBlock.csv";
            if(isset($_SERVER['HTTP_USER_AGENT']) && strpos($_SERVER['HTTP_USER_AGENT'],'MSIE')) {
                header('Pragma: public');
                header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
            }
            header('Content-Type: application/csv');
            header('Content-disposition: attachment; filename="'.$fileName.'"');
            echo $header;
            
            foreach ($data as $key =>$row){
                $exportData = array();
                $exportData[] = $row['#_crm_label_#'];
                foreach ($listFieldsName as $fieldName){
                    $exportData[] = $row[$fieldName];
                }
                $exportData[] = implode($row['ID'],'x');
                $line = implode("\",\"",$exportData);
                $line = "\"" .$line;
                $line .= "\"\r\n";
                echo $line;
            }
        }
    }
    private function getFieldsOfBlock($tableBlockId,$sourceModule){
        global $adb;
        $fields = array();
        $sql = "SELECT fieldname FROM vtiger_field
                INNER JOIN table_block_fields ON table_block_fields.field = vtiger_field.fieldid
                WHERE table_block_fields.tableblockid = ?
                ORDER BY table_block_fields.sequence ASC";
        $rs = $adb->pquery($sql,array($tableBlockId));
        $moduleInstance = Vtiger_Module_Model::getInstance($sourceModule);
        while ($rowField = $adb->fetchByAssoc($rs)){
            $fields[$rowField['fieldname']] = Vtiger_Field_Model::getInstance($rowField['fieldname'],$moduleInstance);
        }
        return $fields;
    }
    private function sanitizeValue($value,$fieldModel,$fieldname){
        global $current_user, $adb;
        $uitype = $fieldModel->get('uitype');
        $fieldType = $fieldModel->getFieldDataType();
        if($uitype == 15 || $uitype == 16 || $uitype == 33){
            $value = str_replace('|##|',',',$value);
            $value = trim($value);
        }elseif($fieldType == 'reference'){
            //have to handle uitype 10
            $value = trim($value);
            if(!empty($value)) {
                $parent_module = getSalesEntityType($value);
                $displayValueArray = getEntityName($parent_module, $value);
                if(!empty($displayValueArray)){
                    foreach($displayValueArray as $k=>$v){
                        $displayValue = $v;
                    }
                }
                if(!empty($parent_module) && !empty($displayValue)){
                    $value = $displayValue;
                }else{
                    $value = "";
                }
            } else {
                $value = '';
            }
        }elseif($uitype == 53){
            $value = getOwnerName($value);
        } elseif($uitype == 71) {
            $value = CurrencyField::convertToUserFormat($value);
        } elseif($uitype == 72) {
            $value = CurrencyField::convertToUserFormat($value, null, true, true);
        } elseif($uitype == 7 || $uitype == 9){
            $value = decimalFormat($value);
        }
        return $value;
    }
}